@@ -18,9 +18,11 @@ import java.lang.reflect.Array;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.ListIterator;
 import java.util.Map;
@@ -211,10 +213,14 @@ public class CollectionUtil
 
     /**
      *  Returns the last element of the passed list, <code>null</code> if
-     *  the list is empty or null.
+     *  the list is empty or null. Uses an indexed get unless the list is
+     *  a subclass of <code>java.util.LinkedList</code>
      */
     public static <T> T last(List<T> list) {
-        return isNotEmpty(list) ? list.get(list.size() - 1) : null;
+        if (isEmpty(list))              return null;
+
+        if (list instanceof LinkedList<?>) return ((LinkedList<T>)list).getLast();
+        return list.get(list.size() - 1);
     }
 
 
@@ -443,6 +449,28 @@ public class CollectionUtil
     }
 
 
+    /**
+     *  Returns <code>true</code> if the passed map is either <code>null</code>
+     *  or has size 0.
+     */
+    public static boolean isEmpty(Map<?,?> m)
+    {
+        return (m == null)
+             ? true
+             : (m.size() == 0);
+    }
+
+
+    /**
+     *  Returns <code>true</code> if the passed map is not <code>null</code>
+     *  and has size &gt; 0.
+     */
+    public static boolean isNotEmpty(Map<?,?> m)
+    {
+        return (m != null) && (m.size() > 0);
+    }
+
+
     /**
      *  Compares two collections of <code>Comparable</code> elements. The two collections are
      *  iterated, and the first not-equal <code>compareTo()</code> result is returned. If the
@@ -763,6 +791,70 @@ public class CollectionUtil
     }
 
 
+    /**
+     *  Partitions the passed iterable into N sublists, each of which has
+     *  at most <code>maxSize</code> elements.
+     */
+    public static <T> List<List<T>> partition(Iterable<T> source, int maxSize)
+    {
+        if (source == null) return Collections.emptyList();
+
+        List<List<T>> result = new ArrayList<List<T>>();
+        List<T> sublist = new ArrayList<T>(maxSize);
+        int count = 0;
+        for (T item : source)
+        {
+            sublist.add(item);
+            count++;
+            if (count >= maxSize)
+            {
+                result.add(sublist);
+                sublist = new ArrayList<T>(maxSize);
+                count = 0;
+            }
+        }
+        if (sublist.size() > 0)
+        {
+            result.add(sublist);
+        }
+        return result;
+    }
+
+
+    /**
+     *  Returns a map that contains all keys in the specified collection.
+     *  <p>
+     *  The returned map is a HashMap; see variant for choosing map type.
+     */
+    public static <K,V> Map<K,V> submap(Map<K,V> src, Collection<K> keys)
+    {
+        return submap(src, keys, new HashMap<K,V>());
+    }
+
+
+    /**
+     *  Extracts all mappings from the source map that correspond to the passed
+     *  keys, and stores them in the destination map. Returns the destination
+     *  map as a convenience.
+     */
+    public static <K,V> Map<K,V> submap(Map<K,V> src, Collection<K> keys, Map<K,V> dest)
+    {
+        if ((src == null) || (keys == null) || (dest == null))
+        {
+            return dest;
+        }
+
+        for (K key : keys)
+        {
+            if (src.containsKey(key))
+            {
+                dest.put(key, src.get(key));
+            }
+        }
+        return dest;
+    }
+
+
 //----------------------------------------------------------------------------
 //  Supporting Objects
 //----------------------------------------------------------------------------
