@@ -49,6 +49,32 @@ implements Map<K,Long>, Iterable<Map.Entry<K,Long>>
 {
     private ConcurrentHashMap<K,AtomicLong> _map = new ConcurrentHashMap<K,AtomicLong>();
 
+//----------------------------------------------------------------------------
+//  Object overrides
+//----------------------------------------------------------------------------
+
+    /**
+     *  Outputs all counters in the format "[ NAME: VALUE, ...]".
+     */
+    @Override
+    public String toString()
+    {
+        StringBuilder sb = new StringBuilder(16384);
+        sb.append("[");
+        for (Map.Entry<K,AtomicLong> entry : _map.entrySet())
+        {
+            if (sb.length() > 1)
+                sb.append(", ");
+
+            sb.append(String.valueOf(entry.getKey()))
+              .append(": ")
+              .append(entry.getValue().longValue());
+        }
+        sb.append("]");
+        return sb.toString();
+    }
+
+
 
 //----------------------------------------------------------------------------
 //  Implementation of Map
@@ -166,13 +192,17 @@ implements Map<K,Long>, Iterable<Map.Entry<K,Long>>
 
 
     /**
-     *  Returns the current keys in the map. This operation is performed directly on the
-     *  underlying map, so does not involve a performance penalty (other than what the
-     *  map incurs).
+     *  Returns the current keys in the map. Note that this represents a point-in-time
+     *  view of the map, so if you are concurrently adding or removing counters it may
+     *  be missing keys or contain keys that are no longer in the map.
      */
     public Set<K> keySet()
     {
-        return _map.keySet();
+        // in Java 8, ConcurrentHashMap broke binary compatibility for keySet(), returning
+        // a different concrete class; this cast forces the compiler to use an invokeinterface
+        // rather than invokevirtual, so the bytecode remains compatible
+
+        return ((Map<K,AtomicLong>)_map).keySet();
     }
 
 
@@ -208,7 +238,6 @@ implements Map<K,Long>, Iterable<Map.Entry<K,Long>>
         return entries;
     }
 
-
 //----------------------------------------------------------------------------
 //  Additional Public Methods
 //----------------------------------------------------------------------------
@@ -230,7 +259,7 @@ implements Map<K,Long>, Iterable<Map.Entry<K,Long>>
 
 
     /**
-     *  Retrieves the value of the specified key as a primitive. If there is no
+     *  Retrieves the value of the specified key as a primitive long. If there is no
      *  mapping for the key, returns 0.
      */
     public long getLong(K key)
@@ -240,6 +269,19 @@ implements Map<K,Long>, Iterable<Map.Entry<K,Long>>
     }
 
 
+    /**
+     *  Retrieves the value of the specified key as a primitive int. If there is no
+     *  mapping for the key, returns 0. This is only valid if you know that your
+     *  counters will remain in integer range (but often works better with other
+     *  variables in your code).
+     */
+    public int getInt(K key)
+    {
+        AtomicLong mapping = _map.get(key);
+        return (mapping == null) ? 0 : (int)mapping.get();
+    }
+
+
     /**
      *  Sets the mapping to the specified value. This is equivalent to calling
      *  {@link #put}, with the same caveats regarding concurent access.
