@@ -24,6 +24,7 @@ import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.TreeMap;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 
@@ -197,18 +198,40 @@ public class TestCollectionUtil extends TestCase
         List<String> l4 = null;
         assertEquals("first(), list is null",   null, CollectionUtil.first(l4));
         assertEquals("last(),  list is null",   null, CollectionUtil.last(l4));
+
+        // this test is just here for coverage; we don't verify behavior
+        List<String> l5 = new LinkedList<String>(Arrays.asList("foo", "bar", "baz"));
+        assertEquals("first(), LinkedList",     "foo", CollectionUtil.first(l5));
+        assertEquals("last(),  LinkedList",     "baz", CollectionUtil.last(l5));
     }
 
 
     public void testIsEmpty() throws Exception
     {
-        assertTrue(CollectionUtil.isEmpty(null));
-        assertTrue(CollectionUtil.isEmpty(Arrays.asList()));
-        assertFalse(CollectionUtil.isEmpty(Arrays.asList("foo")));
+        List<String> list1 = null;
+        assertTrue(CollectionUtil.isEmpty(list1));
+        assertFalse(CollectionUtil.isNotEmpty(list1));
 
-        assertFalse(CollectionUtil.isNotEmpty(null));
-        assertFalse(CollectionUtil.isNotEmpty(Arrays.asList()));
-        assertTrue(CollectionUtil.isNotEmpty(Arrays.asList("foo")));
+        List<String> list2 = new ArrayList<String>();
+        assertTrue(CollectionUtil.isEmpty(list2));
+        assertFalse(CollectionUtil.isNotEmpty(list2));
+
+        List<String> list3 = Arrays.asList("foo");
+        assertFalse(CollectionUtil.isEmpty(list3));
+        assertTrue(CollectionUtil.isNotEmpty(list3));
+
+        Map<String,String> map1 = null;
+        assertTrue(CollectionUtil.isEmpty(map1));
+        assertFalse(CollectionUtil.isNotEmpty(map1));
+
+        Map<String,String> map2 = new HashMap<String,String>();
+        assertTrue(CollectionUtil.isEmpty(map2));
+        assertFalse(CollectionUtil.isNotEmpty(map2));
+
+        Map<String,String> map3 = new HashMap<String,String>();
+        map3.put("foo", "bar");
+        assertFalse(CollectionUtil.isEmpty(list3));
+        assertTrue(CollectionUtil.isNotEmpty(map3));
     }
 
 
@@ -856,4 +879,56 @@ public class TestCollectionUtil extends TestCase
         }
     }
 
+
+    public void testPartition() throws Exception
+    {
+        assertEquals(
+                "null source",
+                Collections.<Integer>emptyList(),
+                CollectionUtil.partition(null, 2));
+
+        assertEquals(
+                "empty source list",
+                Collections.<Integer>emptyList(),
+                CollectionUtil.partition(Collections.<Integer>emptyList(), 2));
+
+        assertEquals(
+                "non-empty source array",
+                Arrays.asList(Arrays.asList(1,2), Arrays.asList(3,4), Arrays.asList(5)),
+                CollectionUtil.partition(Arrays.asList(1,2,3,4,5), 2));
+    }
+
+
+    public void testSubmap() throws Exception
+    {
+        Map<Integer,String> source = new HashMap<Integer,String>();
+        source.put(1, "foo");
+        source.put(2, "bar");
+        source.put(3, "baz");
+
+        assertEquals("null source",
+                     Collections.emptyMap(),
+                     CollectionUtil.submap(null, Arrays.asList(1, 2, 3)));
+
+        assertEquals("null keylist",
+                     Collections.emptyMap(),
+                     CollectionUtil.submap(source, null));
+
+        assertEquals("null destination",
+                     null,
+                     CollectionUtil.submap(source, Arrays.asList(1,2), null));
+
+        Map<Integer,String> expected = new HashMap<Integer,String>();
+        expected.put(2, "bar");
+
+        assertEquals("normal operation",
+                     expected,
+                     CollectionUtil.submap(source, Arrays.asList(2)));
+
+        Map<Integer,String> dest = new TreeMap<Integer,String>();
+
+        assertSame("explicit destination",
+                     dest,
+                     CollectionUtil.submap(source, Arrays.asList(2), dest));
+    }
 }
